@@ -2,7 +2,7 @@
   FILE: icalcomponent.c
   CREATOR: eric 28 April 1999
   
-  $Id: icalcomponent.c 463628 2005-09-24 16:27:03Z winterz $
+  $Id: icalcomponent.c,v 1.64 2008-01-30 20:28:42 dothebart Exp $
 
  (C) COPYRIGHT 2000, Eric Busboom, http://www.softwarestudio.org
 
@@ -281,13 +281,16 @@ icalcomponent_as_ical_string (icalcomponent* impl)
     pvl_elem itr;
 
 /* RFC 2445 explicitly says that the newline is *ALWAYS* a \r\n (CRLF)!!!! */
-char newline[] = "\r\n";
 
 /* WIN32 automatically adds the \r, Anybody else need it?
+   well, the spec says \r\n is a MUST
+
 #ifdef ICAL_UNIX_NEWLINE    
     char newline[] = "\n";
 #else
+*/
     char newline[] = "\r\n";
+/*
 #endif
 */
    
@@ -826,6 +829,7 @@ int icalproperty_recurrence_is_excluded(icalcomponent *comp,
 				       struct icaltimetype *dtstart,
 				       struct icaltimetype *recurtime) {
   icalproperty *exdate, *exrule;
+  pvl_elem property_iterator = comp->property_iterator;
 
   if (comp == NULL || 
       dtstart == NULL || 
@@ -843,14 +847,16 @@ int icalproperty_recurrence_is_excluded(icalcomponent *comp,
 
     if (icaltime_compare(*recurtime, exdatetime) == 0) {
       /** MATCHED **/
+        
+      comp->property_iterator = property_iterator;
       return 1;
     }
   }
 
   /** Now test against the EXRULEs **/
   for (exrule = icalcomponent_get_first_property(comp,ICAL_EXRULE_PROPERTY);
-       exdate != NULL;
-       exdate = icalcomponent_get_next_property(comp,ICAL_EXRULE_PROPERTY)) {
+       exrule != NULL;
+       exrule = icalcomponent_get_next_property(comp,ICAL_EXRULE_PROPERTY)) {
 	 
     struct icalrecurrencetype recur = icalproperty_get_exrule(exrule);
     icalrecur_iterator *exrule_itr  = icalrecur_iterator_new(recur, *dtstart);
@@ -866,6 +872,7 @@ int icalproperty_recurrence_is_excluded(icalcomponent *comp,
       result = icaltime_compare(*recurtime, exrule_time);
       if (result == 0) {
 	icalrecur_iterator_free(exrule_itr);
+        comp->property_iterator = property_iterator;
 	return 1; /** MATCH **/
       }
       if (result == 1)
@@ -874,6 +881,7 @@ int icalproperty_recurrence_is_excluded(icalcomponent *comp,
 
     icalrecur_iterator_free(exrule_itr);
   }
+  comp->property_iterator = property_iterator; 
 
   return 0;  /** no matches **/
 }
@@ -1019,7 +1027,7 @@ void icalcomponent_foreach_recurrence(icalcomponent* comp,
     struct icaltimetype rrule_time = icalrecur_iterator_next(rrule_itr);
     /** note that icalrecur_iterator_next always returns dtstart
 	the first time.. **/
-    
+
     while (1) {
       rrule_time = icalrecur_iterator_next(rrule_itr);
 
@@ -1240,7 +1248,7 @@ struct icalcomponent_kind_map {
 
   
 
-static struct icalcomponent_kind_map component_map[] = 
+static const struct icalcomponent_kind_map component_map[] = 
 {
     { ICAL_VEVENT_COMPONENT, "VEVENT" },
     { ICAL_VTODO_COMPONENT, "VTODO" },
@@ -1256,9 +1264,10 @@ static struct icalcomponent_kind_map component_map[] =
     { ICAL_VSCHEDULE_COMPONENT, "SCHEDULE" },
 
     /* CAP components */
-    { ICAL_VQUERY_COMPONENT, "VQUERY" },  
     { ICAL_VCAR_COMPONENT, "VCAR" },  
     { ICAL_VCOMMAND_COMPONENT, "VCOMMAND" },  
+    { ICAL_VQUERY_COMPONENT, "VQUERY" },  
+    { ICAL_VREPLY_COMPONENT, "VREPLY" },
 
     /* libical private components */
     { ICAL_XLICINVALID_COMPONENT, "X-LIC-UNKNOWN" },  
@@ -1484,7 +1493,7 @@ icalproperty_method icalcomponent_get_method(icalcomponent* comp)
  */
 void icalcomponent_set_dtstart(icalcomponent* comp, struct icaltimetype v)
 {
-    char *tzid;
+    const char *tzid;
     ICALSETUPSET(ICAL_DTSTART_PROPERTY);
 
     if (prop == 0){
@@ -1513,6 +1522,7 @@ void icalcomponent_set_dtstart(icalcomponent* comp, struct icaltimetype v)
 static struct icaltimetype
 icalcomponent_get_datetime(icalcomponent *comp, icalproperty *prop) {
 
+    icalcomponent      *c;
     icalparameter      *param;
     struct icaltimetype	ret;
 
@@ -1521,11 +1531,19 @@ icalcomponent_get_datetime(icalcomponent *comp, icalproperty *prop) {
     if ((param = icalproperty_get_first_parameter(prop, ICAL_TZID_PARAMETER))
 	!= NULL) {
 	const char     *tzid = icalparameter_get_tzid(param);
-	icaltimezone   *tz;
+	icaltimezone   *tz = NULL;
 
-	if ((tz = icalcomponent_get_timezone(comp, tzid)) != NULL) {
-	    icaltime_set_timezone(&ret, tz);
+	for (c = comp; c != NULL; c = icalcomponent_get_parent(c)) {
+	    tz = icalcomponent_get_timezone(c, tzid);
+	    if (tz != NULL)
+		break;
 	}
+
+	if (tz == NULL)
+	    tz = icaltimezone_get_builtin_timezone_from_tzid(tzid);
+
+	if (tz != NULL)
+	    ret = icaltime_set_timezone(&ret, tz);
     }
 
     return ret;
@@ -1602,7 +1620,7 @@ struct icaltimetype icalcomponent_get_dtend(icalcomponent* comp)
  */
 void icalcomponent_set_dtend(icalcomponent* comp, struct icaltimetype v)
 {
-    char *tzid;
+    const char *tzid;
     ICALSETUPSET(ICAL_DTEND_PROPERTY);
 
     if (icalcomponent_get_first_property(inner,ICAL_DURATION_PROPERTY)
@@ -2035,6 +2053,10 @@ icalcomponent* icalcomponent_new_vquery()
 {
     return icalcomponent_new(ICAL_VQUERY_COMPONENT);
 }
+icalcomponent* icalcomponent_new_vreply()
+{
+    return icalcomponent_new(ICAL_VREPLY_COMPONENT);
+}
 
 /*
  * Timezone stuff.
@@ -2051,7 +2073,7 @@ void icalcomponent_merge_component(icalcomponent* comp,
 {
   icalcomponent *subcomp, *next_subcomp;
   icalarray *tzids_to_rename;
-  int i;
+  unsigned int i;
 
   /* Check that both components are VCALENDAR components. */
   assert (icalcomponent_isa(comp) == ICAL_VCALENDAR_COMPONENT);
@@ -2171,6 +2193,7 @@ icalcomponent_handle_conflicting_vtimezones (icalcomponent *comp,
   int i, suffix, max_suffix = 0, num_elements;
   unsigned int tzid_len;
   char *tzid_copy, *new_tzid, suffix_buf[32];
+  (void)tzid_prop; /* hack to stop unused variable warning */
 
   /* Find the length of the TZID without any trailing digits. */
   tzid_len = icalcomponent_get_tzid_prefix_len (tzid);
@@ -2185,7 +2208,8 @@ icalcomponent_handle_conflicting_vtimezones (icalcomponent *comp,
   num_elements = comp->timezones ? comp->timezones->num_elements : 0;
   for (i = 0; i < num_elements; i++) {
     icaltimezone *zone;
-    char *existing_tzid, *existing_tzid_copy;
+    const char *existing_tzid;
+    const char *existing_tzid_copy;
     unsigned int existing_tzid_len;
 
     zone = icalarray_element_at (comp->timezones, i);
@@ -2282,7 +2306,7 @@ static void icalcomponent_rename_tzids_callback(icalparameter *param, void *data
 
     /* Step through the rename table to see if the current TZID matches
        any of the ones we want to rename. */
-    for (i = 0; i < rename_table->num_elements - 1; i += 2) {
+    for (i = 0; (unsigned int)i < rename_table->num_elements - 1; i += 2) {
         if (!strcmp (tzid, icalarray_element_at (rename_table, i))) {
 	    icalparameter_set_tzid (param, icalarray_element_at (rename_table, i + 1));
 	    break;
@@ -2339,7 +2363,7 @@ icaltimezone* icalcomponent_get_timezone(icalcomponent* comp, const char *tzid)
 {
     icaltimezone *zone;
     int lower, upper, middle, cmp;
-    char *zone_tzid;
+    const char *zone_tzid;
 
     if (!comp->timezones)
 	return NULL;
