@@ -3,7 +3,7 @@
   FILE: icalparser.c
   CREATOR: eric 04 August 1999
   
-  $Id: icalparser.c 460715 2005-09-14 21:55:04Z winterz $
+  $Id: icalparser.c,v 1.53 2008-01-15 23:17:40 dothebart Exp $
   $Locker:  $
     
  The contents of this file are subject to the Mozilla Public License
@@ -30,7 +30,8 @@
 
   The Initial Developer of the Original Code is Eric Busboom
 
- (C) COPYRIGHT 2000, Eric Busboom, http://www.softwarestudio.org
+ (C) COPYRIGHT 2000, Eric Busboom <eric@softwarestudio.org>
+     http://www.softwarestudio.org
  ======================================================================*/
 
 #ifdef HAVE_CONFIG_H
@@ -49,10 +50,16 @@
 #include <string.h> /* For strncpy & size_t */
 #include <stdio.h> /* For FILE and fgets and snprintf */
 #include <stdlib.h> /* for free */
+#include <ctype.h>
 
 #include "icalmemory.h"
 #include "icalparser.h"
 
+#ifdef WIN32
+#define HAVE_CTYPE_H
+#define HAVE_ISWSPACE
+#endif
+
 #ifdef HAVE_WCTYPE_H
 # include <wctype.h>
 /* Some systems have an imcomplete implementation on wctype (FreeBSD,
@@ -61,7 +68,7 @@
 #  define iswspace        isspace
 # endif
 #else
-# ifndef HAVE_ISWSPACE
+# if !defined HAVE_ISWSPACE && !defined WIN32
 #  define iswspace        isspace
 # endif
 #endif
@@ -71,11 +78,11 @@
 #define strcasecmp    stricmp
 #endif
 
-char* icalparser_get_next_char(char c, char *str, int qm);
-char* icalparser_get_next_parameter(char* line,char** end);
-char* icalparser_get_next_value(char* line, char **end, icalvalue_kind kind);
-char* icalparser_get_prop_name(char* line, char** end);
-char* icalparser_get_param_name(char* line, char **end);
+static char* parser_get_next_char(char c, char *str, int qm);
+static char* parser_get_next_parameter(char* line,char** end);
+static char* parser_get_next_value(char* line, char **end, icalvalue_kind kind);
+static char* parser_get_prop_name(char* line, char** end);
+static char* parser_get_param_name(char* line, char **end, char **buf_value);
 
 #define TMP_BUF_SIZE 80
 
@@ -98,6 +105,18 @@ struct icalparser_impl
 };
 
 
+char* strstrip (char *str)
+{
+	if (!str)
+		return NULL;
+	while (*str == ' ')
+		str = str+1;
+	while (*str && str[strlen  (str)-1] == ' ')
+		str[strlen (str)-1] = 0;
+	return str;
+}
+
+
 icalparser* icalparser_new(void)
 {
     struct icalparser_impl* impl = 0;
@@ -150,8 +169,8 @@ icalvalue* icalvalue_new_From_string_with_error(icalvalue_kind kind,
                                                 icalproperty **error);
 
 
-
-char* icalparser_get_next_char(char c, char *str, int qm)
+static
+char* parser_get_next_char(char c, char *str, int qm)
 {
     int quote_mode = 0;
     char* p;
@@ -185,14 +204,16 @@ static char* make_segment(char* start, char* end)
     char *buf, *tmp;
     size_t size = (size_t)end - (size_t)start;
     
-    buf = icalmemory_tmp_buffer(size+1);
+    /*buf = icalmemory_tmp_buffer(size+1);*/
     
+    buf = icalmemory_new_buffer(size+1);
 
     strncpy(buf,start,size);
     *(buf+size) = 0;
 
 	tmp = (buf+size);
-	while ( *tmp == '\0' || iswspace(*tmp) )
+	while ((tmp >= buf) && 
+		   ((*tmp == '\0') || iswspace(*tmp)))
 	{
 		*tmp = 0;
 		tmp--;
@@ -201,15 +222,15 @@ static char* make_segment(char* start, char* end)
     return buf;
 }
 
-
-char* icalparser_get_prop_name(char* line, char** end)
+static
+char* parser_get_prop_name(char* line, char** end)
 {
     char* p;
     char* v;
     char *str;
 
-    p = icalparser_get_next_char(';',line,1); 
-    v = icalparser_get_next_char(':',line,1); 
+    p = parser_get_next_char(';',line,1); 
+    v = parser_get_next_char(':',line,1); 
     if (p== 0 && v == 0) {
 	return 0;
     }
@@ -227,14 +248,15 @@ char* icalparser_get_prop_name(char* line, char** end)
     return str;
 }
 
-
-char* icalparser_get_param_name(char* line, char **end)
+static
+char* parser_get_param_name(char* line, char **end, char **buf)
 {
     char* next; 
     char *str;
 
-    next = icalparser_get_next_char('=',line,1);
+    next = parser_get_next_char('=',line,1);
 
+    *buf = 0;
     if (next == 0) {
 	return 0;
     }
@@ -243,24 +265,25 @@ char* icalparser_get_param_name(char* line, char **end)
     *end = next+1;
     if (**end == '"') {
         *end = *end+1;
-	    next = icalparser_get_next_char('"',*end,0);
+	    next = parser_get_next_char('"',*end,0);
 	    if (next == 0) {
 		    return 0;
 	    }
 
-	    *end = make_segment(*end,next);
+	    *buf = *end = make_segment(*end,next);
     }
 
     return str;
 }
 
-
-char* icalparser_get_next_paramvalue(char* line, char **end)
+#if 0
+static
+char* parser_get_next_paramvalue(char* line, char **end)
 {
     char* next; 
     char *str;
 
-    next = icalparser_get_next_char(',',line,1);
+    next = parser_get_next_char(',',line,1);
 
     if (next == 0){
 	next = (char*)(size_t)line+(size_t)strlen(line);\
@@ -274,7 +297,7 @@ char* icalparser_get_next_paramvalue(char* line, char **end)
 	return str;
     }
 }
-
+#endif
 
 /**
    A property may have multiple values, if the values are seperated by
@@ -282,7 +305,8 @@ char* icalparser_get_next_paramvalue(char* line, char **end)
    comma after line and will set the next place to start searching in
    end. */
 
-char* icalparser_get_next_value(char* line, char **end, icalvalue_kind kind)
+static 
+char* parser_get_next_value(char* line, char **end, icalvalue_kind kind)
 {
     
     char* next;
@@ -293,10 +317,10 @@ char* icalparser_get_next_value(char* line, char **end, icalvalue_kind kind)
     p = line;
     while(1){
 
-	next = icalparser_get_next_char(',',p,1);
+	next = parser_get_next_char(',',p,1);
 
 	/* Unforunately, RFC2445 says that for the RECUR value, COMMA
-	   can both seperate digits in a list, and it can seperate
+	   can both separate digits in a list, and it can separate
 	   multiple recurrence specifications. This is not a friendly
 	   part of the spec. This weirdness tries to
 	   distinguish the two uses. it is probably a HACK*/
@@ -306,7 +330,7 @@ char* icalparser_get_next_value(char* line, char **end, icalvalue_kind kind)
 		 (*end+length) > next+5 &&
 		 strncmp(next,"FREQ",4) == 0
 		) {
-		/* The COMMA was followed by 'FREQ', is it a real seperator*/
+		/* The COMMA was followed by 'FREQ', is it a real separator*/
 		/* Fall through */
 	    } else if (next != 0){
 		/* Not real, get the next COMMA */
@@ -326,7 +350,7 @@ char* icalparser_get_next_value(char* line, char **end, icalvalue_kind kind)
 	}
 
 	/* If the comma is preceded by a '\', then it is a literal and
-	   not a value seperator*/  
+	   not a value separator*/  
       
 	if ( (next!=0 && *(next-1) == '\\') ||
 	     (next!=0 && *(next-3) == '\\')
@@ -357,20 +381,21 @@ char* icalparser_get_next_value(char* line, char **end, icalvalue_kind kind)
    
 }
 
-char* icalparser_get_next_parameter(char* line,char** end)
+static 
+char* parser_get_next_parameter(char* line,char** end)
 {
     char *next;
     char *v;
     char *str;
 
-    v = icalparser_get_next_char(':',line,1); 
-    next = icalparser_get_next_char(';', line,1);
+    v = parser_get_next_char(':',line,1); 
+    next = parser_get_next_char(';', line,1);
     
     /* There is no ';' or, it is after the ':' that marks the beginning of
        the value */
 
     if (next == 0 || next > v) {
-	next = icalparser_get_next_char(':', line,1);
+	next = parser_get_next_char(':', line,1);
     }
 
     if (next != 0) {
@@ -471,8 +496,8 @@ char* icalparser_get_line(icalparser *parser,
 	
  
 	/* If the output line ends in a '\n' and the temp buffer
-	   begins with a ' ', then the buffer holds a continuation
-	   line, so keep reading.  */
+	   begins with a ' ' or tab, then the buffer holds a continuation
+	   line, so keep reading.  RFC 2445, section 4.1 */
 
 	if  ( line_p > line+1 && *(line_p-1) == '\n'
 		  && (parser->temp[0] == ' ' || parser->temp[0] == '\t') ) {
@@ -513,8 +538,8 @@ char* icalparser_get_line(icalparser *parser,
 
 }
 
-static void insert_error(icalcomponent* comp, char* text, 
-		  char* message, icalparameter_xlicerrortype type)
+static void insert_error(icalcomponent* comp, const char* text,
+		  const char* message, icalparameter_xlicerrortype type)
 {
     char temp[1024];
     
@@ -599,7 +624,7 @@ icalcomponent* icalparser_parse(icalparser *parser,
         }
 	cont = 0;
 	if(line != 0){
-	    free(line);
+	    icalmemory_free_buffer(line);
 		cont = 1;
 	}
     } while ( cont );
@@ -615,6 +640,7 @@ icalcomponent* icalparser_add_line(icalparser* parser,
                                        char* line)
 { 
     char *str;
+	char *strptr;
     char *end;
     int vcount = 0;
     icalproperty *prop;
@@ -642,9 +668,9 @@ icalcomponent* icalparser_add_line(icalparser* parser,
        a component */
 
     end = 0;
-    str = icalparser_get_prop_name(line, &end);
+    str = parser_get_prop_name(line, &end);
 
-    if (str == 0 || strlen(str) == 0 ){
+    if (str == 0 || *str == '\0' ){
 	/* Could not get a property name */
 	icalcomponent *tail = pvl_data(pvl_tail(parser->components));
 
@@ -655,12 +681,13 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 	}
 	tail = 0;
 	parser->state = ICALPARSER_ERROR;
+	icalmemory_free_buffer(str), str = NULL;
 	return 0; 
     }
 
     /**********************************************************************
      * Handle begin and end of components
-     **********************************************************************/								       
+     **********************************************************************/
     /* If the property name is BEGIN or END, we are actually
        starting or ending a new component */
 
@@ -669,9 +696,10 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 	icalcomponent *c;
         icalcomponent_kind comp_kind;
 
+	icalmemory_free_buffer(str), str = NULL;
 
 	parser->level++;
-	str = icalparser_get_next_value(end,&end, value_kind);
+	str = parser_get_next_value(end,&end, value_kind);
 	    
 
         comp_kind = icalenum_string_to_component_kind(str);
@@ -696,13 +724,17 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 	pvl_push(parser->components,c);
 
 	parser->state = ICALPARSER_BEGIN_COMP;
+
+	icalmemory_free_buffer(str), str = NULL;
 	return 0;
 
     } else if (strcasecmp(str,"END") == 0 ) {
 	icalcomponent* tail;
 
+	icalmemory_free_buffer(str), str = NULL;
 	parser->level--;
-	str = icalparser_get_next_value(end,&end, value_kind);
+
+	str = parser_get_next_value(end,&end, value_kind);
 
 	/* Pop last component off of list and add it to the second-to-last*/
 	parser->root_component = pvl_pop(parser->components);
@@ -714,6 +746,7 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 	} 
 
 	tail = 0;
+	icalmemory_free_buffer(str), str = NULL;
 
 	/* Return the component if we are back to the 0th level */
 	if (parser->level == 0){
@@ -745,6 +778,7 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 
     if(pvl_data(pvl_tail(parser->components)) == 0){
 	parser->state = ICALPARSER_ERROR;
+	icalmemory_free_buffer(str), str = NULL;
 	return 0;
     }
 
@@ -783,9 +817,12 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 	    
 	tail = 0;
 	parser->state = ICALPARSER_ERROR;
+	icalmemory_free_buffer(str), str = NULL;
 	return 0;
     }
 
+    icalmemory_free_buffer(str), str = NULL;
+
     /**********************************************************************
      * Handle parameter values
      **********************************************************************/								       
@@ -795,23 +832,25 @@ icalcomponent* icalparser_add_line(icalparser* parser,
     while(1) {
 
 	if (*(end-1) == ':'){
-	    /* if the last seperator was a ":" and the value is a
+	    /* if the last separator was a ":" and the value is a
 	       URL, icalparser_get_next_parameter will find the
 	       ':' in the URL, so better break now. */
 	    break;
 	}
 
-	str = icalparser_get_next_parameter(end,&end);
-
+	str = parser_get_next_parameter(end,&end);
+	strptr = str;
+	str = strstrip (str);
 	if (str != 0){
 	    char* name;
 	    char* pvalue;
+		char *buf_value = NULL;
         
 	    icalparameter *param = 0;
 	    icalparameter_kind kind;
 	    icalcomponent *tail = pvl_data(pvl_tail(parser->components));
 
-	    name = icalparser_get_param_name(str,&pvalue);
+	    name = parser_get_param_name(str,&pvalue,&buf_value);
 
 	    if (name == 0){
 		/* 'tail' defined above */
@@ -830,10 +869,13 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 		    icalparameter_set_xname(param,name);
 		    icalparameter_set_xvalue(param,pvalue);
 		}
-
+		icalmemory_free_buffer(buf_value), buf_value = NULL;
 
 	    } else if (kind != ICAL_NO_PARAMETER){
 		param = icalparameter_new_from_value_string(kind,pvalue);
+
+		icalmemory_free_buffer(buf_value), buf_value = NULL;
+
 	    } else {
 		/* Error. Failed to parse the parameter*/
 		/* 'tail' defined above */
@@ -851,6 +893,11 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 		    
 		tail = 0;
 		parser->state = ICALPARSER_ERROR;
+		
+		icalmemory_free_buffer(buf_value), buf_value = NULL;
+		icalmemory_free_buffer(name), name = NULL;
+		icalmemory_free_buffer(strptr), strptr = str = NULL;
+  	  	
 		continue;
 	    }
 
@@ -881,17 +928,24 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 		    icalparameter_free(param);
 		    tail = 0;
 		    parser->state = ICALPARSER_ERROR;
-		    return 0;
+
+			icalmemory_free_buffer(name), name = NULL;
+			icalmemory_free_buffer(strptr), strptr = str = NULL;
+		    continue;
 		} 
 	    }
+		icalmemory_free_buffer(name), name = NULL;
 
 	    /* Everything is OK, so add the parameter */
 	    icalproperty_add_parameter(prop,param);
 	    tail = 0;
+		icalmemory_free_buffer(strptr), strptr = str = NULL;
 
 	} else { /* if ( str != 0)  */
 	    /* If we did not get a param string, go on to looking
 	       for a value */
+			if (strptr != NULL)
+					icalmemory_free_buffer(strptr), strptr = str = NULL;
 	    break;
 	} /* if ( str != 0)  */
 
@@ -907,7 +961,9 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 
     vcount=0;
     while(1) {
-	str = icalparser_get_next_value(end,&end, value_kind);
+	str = parser_get_next_value(end,&end, value_kind);
+	strptr = str;
+	str = strstrip (str);
 
 	if (str != 0){
 		
@@ -943,15 +999,20 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 		prop = 0;
 		tail = 0;
 		parser->state = ICALPARSER_ERROR;
+	
+		icalmemory_free_buffer(strptr), str = NULL, strptr = NULL;
 		return 0;
 		    
 	    } else {
 		vcount++;
 		icalproperty_set_value(prop, value);
 	    }
-
+ 	    icalmemory_free_buffer(strptr), str = NULL, strptr = NULL;
 
 	} else {
+			if (strptr != NULL)
+					icalmemory_free_buffer(strptr), str = NULL, strptr = NULL;
+			
 	    if (vcount == 0){
 		char temp[200]; /* HACK */
 		
@@ -1020,7 +1081,7 @@ icalcomponent* icalparser_clean(icalparser* parser)
 	parser->root_component = pvl_pop(parser->components);
 	tail=pvl_data(pvl_tail(parser->components));
 
-	if(tail != 0){
+	if(tail != 0 && parser->root_component != NULL){
 	    if(icalcomponent_get_parent(parser->root_component)!=0){
 		icalerror_warn("icalparser_clean is trying to attach a component for the second time");
 	    } else {
