@@ -3,10 +3,11 @@
     FILE: icalclassify.c
     CREATOR: ebusboom 23 aug 2000
   
-    $Id: icalclassify.c 463628 2005-09-24 16:27:03Z winterz $
+    $Id: icalclassify.c,v 1.19 2008-01-15 23:17:43 dothebart Exp $
     $Locker:  $
     
-    (C) COPYRIGHT 2000, Eric Busboom, http://www.softwarestudio.org
+    (C) COPYRIGHT 2000, Eric Busboom <eric@softwarestudio.org>
+        http://www.softwarestudio.org
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of either: 
@@ -26,9 +27,10 @@
 #include "config.h"
 #endif
 
-#include "ical.h"
+#include <libical/ical.h>
 #include "icalclassify.h"
-#include "icalmemory.h"
+#include <libical/icalmemory.h>
+#include "icalerror.h"
 
 #include <ctype.h>   /* For tolower() */
 #include <string.h> /* for index() */
@@ -221,7 +223,10 @@ void icalssutil_get_parts(icalcomponent* c,
 
     p = icalcomponent_get_first_property(inner,ICAL_ORGANIZER_PROPERTY);
     if(p!=0){
-	parts->organizer = strdup(icalproperty_get_organizer(p));
+      const char *p_organizer = icalproperty_get_organizer(p);
+      if (p_organizer!=0) {
+        parts->organizer = strdup(p_organizer);
+      }
     }
 
     p = icalcomponent_get_first_property(inner,ICAL_SEQUENCE_PROPERTY);    
@@ -231,7 +236,10 @@ void icalssutil_get_parts(icalcomponent* c,
 
     p = icalcomponent_get_first_property(inner,ICAL_UID_PROPERTY);
     if(p!=0){
-	parts->uid = strdup(icalproperty_get_uid(p));
+      const char *p_uid = icalproperty_get_uid(p);
+      if (p_uid!=0) {
+        parts->uid = strdup(p_uid);
+      }
     }
 
     p = icalcomponent_get_first_property(inner,ICAL_RECURRENCEID_PROPERTY);
@@ -249,7 +257,7 @@ void icalssutil_get_parts(icalcomponent* c,
 	p  = icalcomponent_get_first_property(inner,ICAL_ATTENDEE_PROPERTY);
 
 	if(p!=0){
-	    char *attendee = 0;
+	    const char *attendee = 0;
 	    param = icalproperty_get_first_parameter(p,ICAL_PARTSTAT_PARAMETER);
 	    
 	    if(param != 0){
@@ -320,6 +328,7 @@ int icalclassify_publish_new(struct icalclassify_parts *comp,
 				const char* user)
 {
     icalclassify_pre;
+    (void)user;
 
     if(comp->method == ICAL_METHOD_PUBLISH &&
 	match == 0 && comp->inner_kind != ICAL_VFREEBUSY_COMPONENT){
@@ -335,6 +344,7 @@ int icalclassify_publish_update(struct icalclassify_parts *comp,
 				const char* user)
 {
     icalclassify_pre;
+    (void)user;
 
     if(comp->method == ICAL_METHOD_PUBLISH &&
 	match !=0 && comp->inner_kind != ICAL_VFREEBUSY_COMPONENT){
@@ -351,6 +361,8 @@ int icalclassify_publish_freebusy(struct icalclassify_parts *comp,
 {
     icalclassify_pre;
 
+    (void)match;
+    (void)user;
     if(comp->method == ICAL_METHOD_PUBLISH &&
        comp->inner_kind == ICAL_VFREEBUSY_COMPONENT){
 	rtrn = 1;
@@ -368,6 +380,7 @@ int icalclassify_request_new(struct icalclassify_parts *comp,
     /* Method is  REQUEST, and there is no match */
 
     icalclassify_pre
+    (void)user;
 
     if(match->c==0 && comp->method == ICAL_METHOD_REQUEST){
 	rtrn = 1;
@@ -386,6 +399,7 @@ int icalclassify_request_update(
        time-related properties are unchanged */
     
     icalclassify_pre
+    (void)user;
 
     if (match != 0 &&
 	comp->sequence >= match->sequence &&
@@ -405,6 +419,7 @@ int icalclassify_request_reschedule(
     /* REQUEST method, Higher SEQUENCE than match, and one or more
        time-related properties are changed */
     icalclassify_pre
+    (void)user;
 
     if (match->c != 0 &&
 	comp->sequence > match->sequence &&
@@ -424,6 +439,7 @@ int icalclassify_request_delegate(
     icalproperty* attendee;
     icalparameter* param;
     icalclassify_pre;
+    (void)match;
 
     attendee = icalclassify_find_attendee(comp->c,user);
 
@@ -450,6 +466,9 @@ int icalclassify_request_new_organizer(
     icalclassify_pre
     icalerror_set_errno(ICAL_UNIMPLEMENTED_ERROR);
     icalclassify_post
+    (void)comp;
+    (void)match;
+    (void)user;
 
 }
 
@@ -461,6 +480,9 @@ int icalclassify_request_status(
     icalclassify_pre
     icalerror_set_errno(ICAL_UNIMPLEMENTED_ERROR);
     icalclassify_post
+    (void)comp;
+    (void)match;
+    (void)user;
 }
 
 int icalclassify_request_forward(
@@ -471,6 +493,9 @@ int icalclassify_request_forward(
     icalclassify_pre
     icalerror_set_errno(ICAL_UNIMPLEMENTED_ERROR);
     icalclassify_post
+    (void)comp;
+    (void)match;
+    (void)user;
 }
 
 int icalclassify_request_freebusy(
@@ -481,6 +506,9 @@ int icalclassify_request_freebusy(
     icalclassify_pre
     icalerror_set_errno(ICAL_UNIMPLEMENTED_ERROR);
     icalclassify_post
+    (void)comp;
+    (void)match;
+    (void)user;
 }
 
 int icalclassify_reply_accept(
@@ -490,6 +518,7 @@ int icalclassify_reply_accept(
 {
     icalproperty* attendee;
     icalclassify_pre;
+    (void)user;
 
     attendee = icalclassify_find_attendee(match->c,comp->reply_attendee);
 
@@ -507,6 +536,7 @@ int icalclassify_reply_decline(
 {
     icalproperty* attendee;
     icalclassify_pre;
+    (void)user;
 
     attendee = icalclassify_find_attendee(match->c,comp->reply_attendee);
 
@@ -524,6 +554,7 @@ int icalclassify_reply_delegate(
 {
     icalproperty* attendee;
     icalclassify_pre;
+    (void)user;
 
     attendee = icalclassify_find_attendee(match->c,comp->reply_attendee);
 
@@ -540,6 +571,7 @@ int icalclassify_reply_crasher_accept(
 {
     icalproperty* attendee;
     icalclassify_pre;
+    (void)user;
 
     attendee= icalclassify_find_attendee(match->c,comp->reply_attendee);
 
@@ -556,6 +588,7 @@ int icalclassify_reply_crasher_decline(
 {
     icalproperty* attendee;
     icalclassify_pre;
+    (void)user;
 
 
     attendee = icalclassify_find_attendee(match->c,comp->reply_attendee);
@@ -572,6 +605,9 @@ int icalclassify_add_instance(
     const char* user)
 {
     icalclassify_pre
+    (void)match;
+    (void)user;
+
     if(comp->method == ICAL_METHOD_ADD){
 	rtrn = 1;
     }
@@ -583,6 +619,8 @@ int icalclassify_cancel_event(
     const char* user)
 {
     icalclassify_pre
+    (void)match;
+    (void)user;
     if(comp->method == ICAL_METHOD_CANCEL){
 	rtrn = 1;
     }
@@ -594,6 +632,8 @@ int icalclassify_cancel_instance(
     const char* user)
 {
     icalclassify_pre
+    (void)match;
+    (void)user;
     if(comp->method == ICAL_METHOD_CANCEL){
 	rtrn = 1;
     }
@@ -605,6 +645,8 @@ int icalclassify_cancel_all(
     const char* user)
 {
     icalclassify_pre
+    (void)match;
+    (void)user;
     if(comp->method == ICAL_METHOD_CANCEL){
 	rtrn = 1;
     }
@@ -616,6 +658,8 @@ int icalclassify_refesh(
     const char* user)
 {
     icalclassify_pre
+    (void)match;
+    (void)user;
     if(comp->method == ICAL_METHOD_REFRESH){
 	rtrn = 1;
     }
@@ -627,6 +671,8 @@ int icalclassify_counter(
     const char* user)
 {
     icalclassify_pre
+    (void)match;
+    (void)user;
     if(comp->method == ICAL_METHOD_COUNTER){
 	rtrn = 1;
     }
@@ -638,6 +684,8 @@ int icalclassify_delinecounter(
     const char* user)
 {
     icalclassify_pre
+    (void)match;
+    (void)user;
 
     if(comp->method == ICAL_METHOD_DECLINECOUNTER){
 	rtrn = 1;
@@ -646,7 +694,7 @@ int icalclassify_delinecounter(
     icalclassify_post
 }
 
-struct icalclassify_map {
+static const struct icalclassify_map {
 	icalproperty_method method;
 	int (*fn)(struct icalclassify_parts *comp,struct icalclassify_parts *match, const char* user);
 	icalproperty_xlicclass class;
