@@ -4,10 +4,11 @@
  CREATOR: eric 21 Aug 2000
 
 
- $Id: icalclassify.h 405592 2005-04-14 19:28:51Z dfaure $
+ $Id: icalclassify.h,v 1.7 2008-01-15 23:17:43 dothebart Exp $
  $Locker:  $
 
- (C) COPYRIGHT 2000, Eric Busboom, http://www.softwarestudio.org
+ (C) COPYRIGHT 2000, Eric Busboom <eric@softwarestudio.org>
+     http://www.softwarestudio.org
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of either: 
@@ -26,8 +27,9 @@
 #ifndef ICALCLASSIFY_H
 #define ICALCLASSIFY_H
 
-#include "ical.h"
-#include "icalset.h"
+#include <libical/ical.h>
+#include <libicalss/icalset.h>
+#include "icalcomponent.h"
 
 icalproperty_xlicclass icalclassify(icalcomponent* c,icalcomponent* match, 
 			      const char* user);
