@@ -3,7 +3,7 @@
   FILE: icalfileset.c
   CREATOR: eric 23 December 1999
   
-  $Id: icalfileset.c 405592 2005-04-14 19:28:51Z dfaure $
+  $Id: icalfileset.c,v 1.36 2008-01-15 23:17:43 dothebart Exp $
   $Locker:  $
     
  (C) COPYRIGHT 2000, Eric Busboom, http://www.softwarestudio.org
@@ -59,7 +59,7 @@
 extern int errno;
 
 /** Default options used when NULL is passed to icalset_new() **/
-icalfileset_options icalfileset_options_default = {O_RDWR|O_CREAT, 0644, 0, 0};
+icalfileset_options icalfileset_options_default = {O_RDWR|O_CREAT, 0644, 0};
 
 int icalfileset_lock(icalfileset *set);
 int icalfileset_unlock(icalfileset *set);
@@ -180,13 +180,13 @@ icalcluster* icalfileset_produce_icalcluster(const char *path) {
 char* icalfileset_read_from_file(char *s, size_t size, void *d)
 {
     char* p = s;
-    int fd = (int)d;
+    icalfileset *set = d;
 
     /* Simulate fgets -- read single characters and stop at '\n' */
 
     for(p=s; p<s+size-1;p++){
 	
-	if(read(fd,p,1) != 1 || *p=='\n'){
+	if(read(set->fd,p,1) != 1 || *p=='\n'){
 	    p++;
 	    break;
 	} 
@@ -209,7 +209,7 @@ icalerrorenum icalfileset_read_file(icalfileset* set,mode_t mode)
   
     parser = icalparser_new();
 
-    icalparser_set_gen_data(parser,(void*)set->fd);
+    icalparser_set_gen_data(parser, set);
     set->cluster = icalparser_parse(parser,icalfileset_read_from_file);
     icalparser_free(parser);
 
@@ -333,7 +333,7 @@ int icalfileset_unlock(icalfileset *set)
 
     return (fcntl(set->fd, F_UNLCK, &lock)); 
 #else
-    return 0;
+	return 0;
 #endif
 }
 
@@ -654,7 +654,7 @@ icalcomponent* icalfileset_fetch_match(icalset* set, icalcomponent *comp)
 icalerrorenum icalfileset_modify(icalset* set, icalcomponent *old,
 				 icalcomponent *new)
 {
-    icalfileset *fset = (icalfileset*) set;
+	// icalfileset *fset = (icalfileset*) set;
 
     assert(0); /* HACK, not implemented */
     return ICAL_NO_ERROR;
@@ -811,7 +811,8 @@ icalsetiter icalfileset_begin_component(icalset* set, icalcomponent_kind kind, i
 
             /* add recurrence-id to the component
                if there is a recurrence-id already, remove it, then add the new one */
-            if (prop = icalcomponent_get_first_property(comp, ICAL_RECURRENCEID_PROPERTY))
+            prop = icalcomponent_get_first_property(comp, ICAL_RECURRENCEID_PROPERTY);
+	    if (prop)
                 icalcomponent_remove_property(comp, prop);
             icalcomponent_add_property(comp, icalproperty_new_recurrenceid(next));
 
@@ -876,7 +877,8 @@ icalcomponent* icalfileset_form_a_matched_recurrence_component(icalsetiter* itr)
 
     /* add recurrence-id to the component
      * if there is a recurrence-id already, remove it, then add the new one */
-    if (prop = icalcomponent_get_first_property(comp, ICAL_RECURRENCEID_PROPERTY))
+    prop = icalcomponent_get_first_property(comp, ICAL_RECURRENCEID_PROPERTY);
+    if (prop)
         icalcomponent_remove_property(comp, prop);
         icalcomponent_add_property(comp, icalproperty_new_recurrenceid(next));
 
@@ -892,7 +894,7 @@ icalcomponent* icalfilesetiter_to_next(icalset* set, icalsetiter* i)
 {
 
     icalcomponent* c = NULL;
-    icalfileset *fset = (icalfileset*) set;
+    /// icalfileset *fset = (icalfileset*) set;
     struct icaltimetype start, next;
     icalproperty *dtstart, *rrule, *prop, *due;
     struct icalrecurrencetype recur;
@@ -945,7 +947,8 @@ icalcomponent* icalfilesetiter_to_next(icalset* set, icalsetiter* i)
 
         /* add recurrence-id to the component
          * if there is a recurrence-id already, remove it, then add the new one */
-        if (prop = icalcomponent_get_first_property(c, ICAL_RECURRENCEID_PROPERTY))
+	prop = icalcomponent_get_first_property(c, ICAL_RECURRENCEID_PROPERTY);
+        if (prop)
             icalcomponent_remove_property(c, prop);
         icalcomponent_add_property(c, icalproperty_new_recurrenceid(next));
 
